/* Copyright 2004 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: main.cpp,v 1.4 2004/10/17 21:42:04 tonic Exp $
 * $Revision: 1.4 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#include "main.h"
#include "GameApp.h"
//#include "CFL.h"


//CFL *gCFL;


static SDL_Surface *sScreen;


static SDL_Surface * setVideoMode(int width, int height, int bpp, int fullscreen, int fsaa)
{
    Uint32 videoFlags = 0;
    int rSize, gSize, bSize;

    switch (bpp)
    {
    case 8:
        rSize = 3;
        gSize = 3;
        bSize = 2;
    case 15:
    case 16:
        rSize = 5;
        gSize = 5;
        bSize = 5;
        break;
    default:
        rSize = 8;
        gSize = 8;
        bSize = 8;
        break;
    }
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, rSize);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, gSize);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, bSize);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    if (fsaa)
    {
        SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
        SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, fsaa);
    }

    videoFlags |= SDL_DOUBLEBUF;
    /* Note: The SDL_OPENGLBLIT isn't recommended for use, but we'll
     *       ignore the advice this time and just do it.
     */
    //videoFlags |= SDL_OPENGL;
    videoFlags |= SDL_OPENGLBLIT;
    if (fullscreen)
        videoFlags |= SDL_FULLSCREEN;
    videoFlags |= SDL_HWSURFACE;

    return SDL_SetVideoMode(width, height, bpp, videoFlags);
}


typedef struct {
    int bpp;
    int fullscreen;
} VIDEO_SETTINGS;

static VIDEO_SETTINGS sVideoSettings[] =
{
#ifdef _DEBUG
    { 32, 0 },
    { 24, 0 },
    { 16, 0 },
#else
    { 32, 1 },
    { 24, 1 },
    { 16, 1 },
    { 32, 0 },
    { 24, 0 },
    { 16, 0 },
    { 32, 1 },
#endif
};
#define SVIDEOSETTINGS_COUNT (sizeof(sVideoSettings) / sizeof(sVideoSettings[0]))


int main(int argc, char *argv[])
{
    int a;

    if (SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO |
                 SDL_INIT_TIMER | SDL_INIT_NOPARACHUTE) < 0)
    {
        errorMessage("Unable to init SDL", SDL_GetError());
        exit(EXIT_FAILURE);
    }
    atexit(SDL_Quit);

    /*
    if ((gCFL = CFL::create("game.dat")) == NULL)
    {
        errorMessage("Game Initialization Error",
                     "Cannot open data storage!\n"
                     "Missing file: game.dat\n"
                     "You may need to reinstall the application.");
        exit(1);
    }
    */

    sScreen = NULL;
    for (a = 0; a < SVIDEOSETTINGS_COUNT; ++a)
    {
        VIDEO_SETTINGS *vs = &sVideoSettings[a];
        INT fs = vs->fullscreen;
        if (argc > 1 && argv[1][0] == 'w')
            fs = 0;
        else if (argc > 1 && argv[1][0] == 'f')
            fs = 1;
        sScreen = setVideoMode(WIDTH, HEIGHT, vs->bpp, fs, 0);
        const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
        if (sScreen != NULL && (sScreen->flags & SDL_OPENGL) && videoInfo && videoInfo->hw_available)
            break;
    }
    if (sScreen == NULL)
    {
        errorMessage("Unable to set video mode", SDL_GetError());
        exit(EXIT_FAILURE);
    }

    GameApp *app = new GameApp;
    BOOL result = app->init();

    initSound(NULL);//gCFL);

    if (result)
        app->run();

    deinitSound();

    //delete gCFL;

    return EXIT_SUCCESS;
}
