/* Turska UI library
 * Copyright 2003-2004 Jetro Lauha
 * All rights reserved.
 * Web: http://iki.fi/jetro/ or http://jet.ro
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of EITHER:
 *   (1) The GNU Lesser General Public License as published by the Free
 *       Software Foundation; either version 2.1 of the License, or (at
 *       your option) any later version. The text of the GNU Lesser
 *       General Public License is included with this library in the
 *       file LICENSE-LGPL.txt.
 *   (2) The BSD-style license that is included with this library in
 *       the file LICENSE-BSD.txt.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the files
 * LICENSE-LGPL.txt and LICENSE-BSD.txt for more details.
 *
 * $Id: UIGraphicsOpenGLSDL.h,v 1.1 2004/08/24 21:54:12 tonic Exp $
 * $Revision: 1.1 $
 */

#ifndef UIGRAPHICSOPENGLSDL_H_INCLUDED
#define UIGRAPHICSOPENGLSDL_H_INCLUDED


struct SDL_Surface;
struct SDL_RWops;


//! OpenGL & SDL using implementation of the UIGraphics interface.
/*! This implementation requires the SDL and SDL_image libraries. 
 */
class UIGraphicsOpenGLSDL : public UIGraphics
{
public:

    UIGraphicsOpenGLSDL() {}
    virtual ~UIGraphicsOpenGLSDL() {}

    // From UIGraphics:

    virtual void beginRender();
    virtual void endRender();

    //! Loads an image using a given file name.
    /*! Images are converted to OpenGL textures and returned as UIImageOpenGL objects.
     */
    virtual UIImage * loadImage(const char *fileName);

    // UIGraphicsOpenGLSDL additions:

    //! Loads an image using SDL read/write operations object.
    /*! Images are converted to OpenGL textures and returned as UIImageOpenGL objects.
     * \param source data object to load image from.
     * \param freeSource TRUE if source should be deleted after loading the image or FALSE otherwise.
     */
    virtual UIImage * loadImage(SDL_RWops *source, BOOL freeSource);

    virtual SDL_Surface * loadSurface(const char *fileName);
    virtual SDL_Surface * loadSurface(SDL_RWops *source, BOOL deleteSource);

    virtual UIImage * createImage(SDL_Surface *surface, UIRectangle *sourceRect = NULL);
    virtual BOOL createTexture(UINT &texture, SDL_Surface *surface, FLOAT *uv, UIRectangle *sourceRect = NULL);
};


#endif // !UIGRAPHICSOPENGLSDL_H_INCLUDED
